/*********************************************
 * OPL 12.6.1.0 Model
 * Author: rpilling
 * Creation Date: 20/04/2015 at 11:10:05 PM
 *********************************************/

//declare sets
{string} DG = ...;
{string} U = ...;
int NbPeriods = ...;
range Periods = 1..NbPeriods;

// declare microgrid input variables 
float mcM[DG] = ...;
float ciM[DG]= ...;
float minpowerM[DG] = ...;
float maxpowerM[DG] = ...;
float demandM[Periods] = ...;
float PVout[Periods] = ...;

// declare utility input variables 
float A[U] = ...;
float B[U]= ...;
float C[U]= ...;
float demandU[Periods] = ...;

// declare microgrid decision variables
dvar float+ xM[DG][Periods];
dvar float+ xU[U][Periods];

//  objective function (capacity)
minimize
  sum(g in DG, t in Periods)(mcM[g]*xM[g][t]) +
  sum(k in U, t in Periods)(1/31)*(A[k]*(xU[k][t])^2 - B[k]*xU[k][t] + C[k]);
  
// constraint equations        
subject to {

// global power balance constraint 
forall (t in Periods)
  ctDemand: sum(g in DG)xM[g][t] + sum(k in U)xU[k][t]  >= demandM[t] + demandU[t] - PVout[t];// balance equation
  
// DG power limit constraints
forall (g in DG, t in Periods)
  ctDGLowerLimit: xM[g][t] >= minpowerM[g];

forall (g in DG, t in Periods)
  ctDGUpperLimit: xM[g][t] <= maxpowerM[g];
}