/*********************************************
 * OPL 12.6.1.0 Model
 * Author: rpilling
 * Creation Date: 20/04/2015 at 11:10:05 PM
 *********************************************/

//declare sets
{string} DG = ...;
{string} ESS = ...;
int NbPeriods = ...;
range Periods = 1..NbPeriods;

// declare distributed generation input variables 
float mcM[DG] = ...;
float fcM[DG] = ...;
float minpowerM[DG] = ...;
float maxpowerM[DG] = ...;
float demandM[Periods] = ...;
float PVout[Periods] = ...;

// declare storage input variables 
float fcS[ESS] = ...;
float maxin[ESS] = ...;
float maxout[ESS] = ...;
float eff_c[ESS] = ...;
float eff_d[ESS] = ...;
float ceS[ESS] = ...;

// declare microgrid decision variables
dvar float+ xM[DG][Periods];

// declare ESS decision variables
dvar float+ eS[ESS][Periods];
dvar float+ inS[ESS][Periods];
dvar float+ outS[ESS][Periods];
dvar float xS[Periods];
dvar int nS[ESS];

//  objective function (with ESS)
minimize
  sum(k in DG, t in Periods)(mcM[k]*xM[k][t]);

// constraint equations        
subject to {

// global power balance constraint 
forall (t in Periods)
  eSOut: sum(s in ESS)(outS[s][t]- inS[s][t]) == xS[t];
  
forall (t in Periods)
  ctDemand: sum(k in DG)xM[k][t] + xS[t] == demandM[t] - PVout[t];// balance equation

// DG power limit constraints
forall (k in DG, t in Periods)
  ctDGLowerLimit: xM[k][t] >= minpowerM[k];

forall (k in DG, t in Periods)
  ctDGUpperLimit: xM[k][t] <= maxpowerM[k];

// storage-specific constraints

forall (s in ESS, t in 2..NbPeriods)
  ctDBalance: eS[s][t] == eS[s][t-1]-(outS[s][t-1]/eff_d[s])+(inS[s][t-1]*eff_c[s]); //Energy charge balance equation
  
forall (s in ESS, t in Periods)
  ctESSLimit: eS[s][t] <= nS[s]*ceS[s]; // energy ESS limit

forall (s in ESS, t in Periods)
  ctChargeLimit: inS[s][t] <= nS[s]*maxin[s]; // charging limit

forall (s in ESS, t in Periods)
  ctDechargeLimit: outS[s][t] <= nS[s]*maxout[s]; // discharging limit
 
 forall (s in ESS)							// terminal equality constraint
  ctInitialPeriod: eS[s][1] == eS[s][24]-(outS[s][24]/eff_d[s])+(inS[s][24]*eff_c[s]);
  
}